#!/usr/bin/env python
# -*- coding:utf-8 -*-
# @Time   : 2019/3/1 16:00
# @Author : yebing
# @Desc : ==============================================
# Life is Short I Use Python!!!                      ===
# If this runs wrong,don't ask me,I don't know why;  ===
# If this runs right,thank god,and I don't know why. ===
# Maybe the answer,my friend,is blowing in the wind. ===
# ======================================================
# @Project : guozhi
# @FileName: ReplyMessage.py

import json
import time
import uuid


class ReplyMessage(object):
    def __init__(self, type_name, code, message, data):
        self.id = str(uuid.uuid1())
        self.type = type_name
        self.data = {
            'code': code,
            'message': message,
            'data': data
        }
        self.time = int(round(time.time() * 1000))

    def __str__(self):
        return json.dumps({
            'id': self.id,
            'type': self.type,
            'time': self.time,
            'data': self.data
        })

    __repr__ = __str__


if __name__ == '__main__':
    aa = ReplyMessage("flag", 200, 'asd', {"gpio_num": 0, "mode": "bcm"})
    print aa



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import Adafruit_DHT

from juice.handler.PropHandler import PropHandler
from juice.util import Const
from juice.util import GpioUtil


class TempGetHandler(PropHandler):
    """
    温度传感器获取，实例化需要设置bcm编号
    """

    def __init__(self, gpio_bcm_num):
        self.gpio = gpio_bcm_num

    """
        传感器温度获取方法
        handle为必须实现的方法
        reply上行消息，返回给服务器
    """

    def handle(self, message):
        self.reply(Const.SUCCESS_CODE, Const.SUCCESS_MESSAGE, self.temp())

    def temp(self):
        sensor = Adafruit_DHT.DHT11
        humidity, temperature = Adafruit_DHT.read_retry(sensor, self.gpio)
        return 'Temp={0:0.1f}*C  Humidity={1:0.1f}%'.format(temperature, humidity)


class SoudGetHandler(PropHandler):
    """
    蜂鸣器状态获取，实例化需要设置bcm编号
    """

    def __init__(self, mode, gpio_num):
        self.mode = mode.lower()
        self.gpio = gpio_num

    """
        传蜂鸣器状态获取方法
        handle为必须实现的方法
        reply上行消息，返回给服务器
    """

    def handle(self, message):
        self.reply(Const.SUCCESS_CODE, Const.SUCCESS_MESSAGE, GpioUtil.GetGPIOStatus(self.mode, self.gpio))



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import json


class IotException(Exception):
    """
        iot异常
    """

    def __init__(self, message, code=200):
        self.code = code
        self.message = message

    def __str__(self):
        return json.dumps({
            'code': self.code,
            'message': self.message
        })



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import json


def readJson(path):
    file = open(path, "rb")
    fileJson = json.load(file)
    product_code = fileJson["product_code"]
    device_code = fileJson["device_code"]
    secret = fileJson["secret"]
    return {'product_code': product_code, 'device_code': device_code, 'secret': secret}



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import json
import warnings

import RPi.GPIO as GPIO

from juice.exception.IotException import IotException

GPIO_STATUS = {-1: 'no mode,please input bcm or board', 0: 'LOW', 1: 'HIGH'}
GPIO_MODE = {'bcm': GPIO.BCM, 'board': GPIO.BOARD}
GPIO_CONST_MODE = 'mode'
GPIO_CONST_NUM = 'gpio_num'
GPIO_CONST_SIGNAL = 'signal'
GPIO_CONST_ACCESS_TYPE = 'access_type'


def checkMode(mode):
    """
    判断模式是否支持
    :param mode:
    :return:
    """
    mode_type = GPIO_MODE.get(mode.lower(), '')
    if mode_type == '':
        raise IotException('mode is error,please input %s or %s' % ('bcm', 'board'), 404)
    else:
        return mode_type


def GetGPIOStatus(mode, gpio_num):
    """
    根据模式和针脚号获得该针脚的状态
    :param mode: 模式，只能是bcm或board
    :param gpio_num: 对应模式的针脚号
    :return: LOW 或 HIGH
    """
    checkMode(mode)

    return json.dumps({
        'mode': mode,
        'gpio_num': gpio_num,
        'access_type': GetGPIOType(mode, gpio_num),
        'signal': GetGPIOStatusNum(mode, gpio_num)
    })


def GetGPIOStatusNum(mode, gpio_num):
    """
    根据模式和针脚号获得该指针的高低电平
    :param mode:
    :param gpio_num:
    :return: -1(无效),0(低),1(高)
    """

    mode_type = checkMode(mode.lower())
    GPIO.setwarnings(True)
    GPIO.setmode(mode_type)
    with warnings.catch_warnings(record=True) as w:
        GPIO.setup(gpio_num, GPIO.gpio_function(gpio_num))
    return GPIO.input(gpio_num)


def GetGPIOType(mode, gpio_num):
    """
    根据模式和针脚号获得目前是输入还是输出
    :param mode:
    :param gpio_num:
    :return:
    """
    mode_type = checkMode(mode.lower())
    GPIO.setwarnings(True)
    GPIO.setmode(mode_type)
    return GPIO.gpio_function(gpio_num)


def initGPIOAccessType(mode, gpio_num, accesstype, signal):
    """
    初始化gpio的接入方式，强制初始化
    :param mode:
    :param gpio_num:
    :param accesstype: 接入类型对应GPIO.in(1),GPIO.out(0)
    :param signal: 高低电平对应GPIO.HIGH(1),GPIO.LOW(0)
    :return:
    """
    mode_type = checkMode(mode.lower())
    GPIO.setwarnings(False)
    GPIO.setmode(mode_type)
    if accesstype == 1:
        # signal取GPIO.PUD_DOWN,GPIO.PUD_UP,GPIO.PUD_OFF
        if signal in [GPIO.PUD_DOWN, GPIO.PUD_UP, GPIO.PUD_OFF]:
            GPIO.setup(gpio_num, accesstype, pull_up_down=signal)
        else:
            raise IotException('GPIO.IN not support signal(GPIO.PUD_OFF(20),GPIO.PUD_DOWN(21), GPIO.PUD_UP(22))',
                               500)
    else:
        # signal取GPIO.IN,GPIO.OUT
        if signal in [GPIO.IN, GPIO.OUT]:
            GPIO.setup(gpio_num, accesstype, initial=signal)
        else:
            raise IotException('GPIO.IN not support signal(GPIO.IN, GPIO.OUT)', 500)


def setGPIOStatus(mode, gpio_num, signal):
    """
    设置gpio针脚电平，当为gpio.input模式，不允许设置
    :param mode:
    :param gpio_num:
    :param signal:
    :return:
    """
    mode_type = checkMode(mode.lower())
    GPIO.setwarnings(True)
    GPIO.setmode(mode_type)
    if GPIO.IN == GPIO.gpio_function(gpio_num):
        raise IotException('access type error,please set GPIO stitch for GPIO.OUT', 400)
    with warnings.catch_warnings(record=True) as w:
        GPIO.setup(gpio_num, GPIO.gpio_function(gpio_num))
        GPIO.output(gpio_num, signal)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# @Time   : 2019/3/1 16:46
# @Author : yebing
# @Desc : ==============================================
# Life is Short I Use Python!!!                      ===
# If this runs wrong,don't ask me,I don't know why;  ===
# If this runs right,thank god,and I don't know why. ===
# Maybe the answer,my friend,is blowing in the wind. ===
# ======================================================
# @Project : guozhi
# @FileName: PropHandler.py

from juice.model.ReplyMessage import ReplyMessage


class PropHandler(object):
    """
        属性获取接口，只需实现handle方法，并调用reply方法即可
    """

    def __init__(self):
        self.id = None
        self.client = None
        self.type = None
        self.topic = None
        self.reply_message = None

    def execute(self, client, topic, message):
        self.id = message.id
        self.client = client
        self.type = message.type
        self.topic = topic
        self.handle(message)
        self._reply()

    def _reply(self):
        if self.reply_message is None:
            raise ValueError('reply message is not None')
        else:
            self.reply_message.id = self.id
            self.client.prop_reply(self.reply_message)

    def reply(self, code=200, message='success', data={}):
        """
        上行消息,不允许重写
        :param code:
        :param message:
        :param data:
        :return:
        """
        self.reply_message = ReplyMessage(self.type + '_reply', code, message, data)

    def handle(self, message):
        """
        云端下发(下行)设置属性消息,需要重写接收flag进行处理
        :param message: JuiceMessage,原始的发送信息属性名称
            'id','type','data','time'
        """
        pass



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from juice.handler.PropHandler import PropSetHandler
from juice.model.JuiceMessage import PropMessage


class SoudSetHandler(PropSetHandler):
    """
        蜂鸣器设置方法
        handle为必须实现的方法
        reply上行消息，返回给服务器
    """

    def handle(self, prop_message):
        self.reply(PropMessage(prop_message.flag, True))



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import os
import time
import uuid


def create_client_id(user_name):
    return user_name + time.strftime('%Y%m%d%H%M%S', time.localtime(time.time()))


def execute_command(shell_command):
    """
    在操作系统上执行命令
    :param shell_command: shell命令
    :return:
    """
    return os.popen(shell_command).readline().strip()


def get_raspberry_prop_info(command):
    """
    获取树莓派系统动态属性
    :param command: 标签命令
    :return:
    """
    command_dict = {
        # 核心电压
        "measure_volts_core": execute_command("vcgencmd measure_volts core | awk -F '=' '{print $2}'"),
        # 温度
        "measure_temp": execute_command("vcgencmd measure_temp | awk -F '=' '{print $2}'"),
        # cpu占内存
        "get_mem_arm": execute_command("vcgencmd get_mem arm | grep arm | awk -F '=' '{print $2}'"),
        # gpu占内存
        "get_mem_gpu": execute_command("vcgencmd get_mem gpu | grep gpu | awk -F '=' '{print $2}'")
    }
    return command_dict.get(command, 'no command')


def get_raspberry_sys_info():
    """
    获取树莓派系统关键信息
    :return:
    """
    return {
        "type": "sys_info",
        "id": str(uuid.uuid1()),
        "data": {
            # 版本代号
            "revision": execute_command("cat /proc/cpuinfo | grep 'Revision' | awk '{print $3}'"),
            # 序列号
            "sn": execute_command("cat /proc/cpuinfo | grep 'Serial' | awk '{print $3}'")
        }
    }



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from juice.handler.SysHandler import SysHandler
from juice.util import Const
from juice.util import Util


class SysArmGetHandler(SysHandler):
    """
        系统CPU内存占比响应方法
        handle为必须实现的方法
    """

    def handle(self, message):
        self.reply(Const.SUCCESS_CODE, Const.SUCCESS_MESSAGE,
                   Util.get_raspberry_prop_info("get_mem_arm"))


class SysGpuGetHandler(SysHandler):
    """
        系统GPU内存占比响应方法
        handle为必须实现的方法
    """

    def handle(self, message):
        self.reply(Const.SUCCESS_CODE, Const.SUCCESS_MESSAGE,
                   Util.get_raspberry_prop_info("get_mem_gpu"))


class SysVoltsGetHandler(SysHandler):
    """
        系统核心电压响应方法
        handle为必须实现的方法
    """

    def handle(self, message):
        self.reply(Const.SUCCESS_CODE, Const.SUCCESS_MESSAGE,
                   Util.get_raspberry_prop_info("measure_volts_core"))


class SysTempGetHandler(SysHandler):
    """
        温度响应方法
        handle为必须实现的方法
    """

    def handle(self, message):
        self.reply(Const.SUCCESS_CODE, Const.SUCCESS_MESSAGE,
                   Util.get_raspberry_prop_info("measure_temp"))



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import os

import sys

reload(sys)
sys.setdefaultencoding('utf8')
from setuptools import setup, find_packages

long_description = ''
if os.path.exists("PIP_README.md"):
    with open("PIP_README.md", "r") as fh:
        long_description = fh.read()
setup(
    name="guozhi",
    version="0.1.6",
    author="yebing",
    author_email="cui_com@qq.com",
    maintainer_email="cui_com@qq.com",
    description="python-iot-sdk for guozhiyun",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://gitee.com/cui_com/jucie-python-sdk.git",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    install_requires=[
        'paho-mqtt>=1.4.0'
        'Adafruit-DHT>=1.4.0'
    ],
    zip_safe=False
)



#!/usr/bin/env python
# -*- coding:utf-8 -*-

from juice.handler.SysHandler import SysHandler
from juice.util import Const
from juice.util import GpioUtil


class GpioInitHandler(SysHandler):
    """
        GPIO针脚初始化
        handle为必须实现的方法
    """

    def handle(self, message):
        if isinstance(message.data, dict):
            status_code = Const.SUCCESS_CODE
            status_message = Const.SUCCESS_MESSAGE
            try:
                GpioUtil.initGPIOAccessType(
                    message.data.get(GpioUtil.GPIO_CONST_MODE, ''), message.data.get(GpioUtil.GPIO_CONST_NUM, -1),
                    message.data.get(GpioUtil.GPIO_CONST_ACCESS_TYPE, ''),
                    message.data.get(GpioUtil.GPIO_CONST_SIGNAL, ''))
            except Exception, err:
                status_code = Const.SERVER_ERROR_CODE
                status_message = 'failure: %s' % err
            self.reply(status_code, status_message)
        else:
            self.reply(Const.SERVER_ERROR_CODE, 'data is not dict')


class GpioSetHandler(SysHandler):
    """
        GPIO针脚电平设置(只支持GPIO.OUT模式)
        handle为必须实现的方法
    """

    def handle(self, message):
        if isinstance(message.data, dict):
            status_code = Const.SUCCESS_CODE
            status_message = Const.SUCCESS_MESSAGE
            try:
                GpioUtil.setGPIOStatus(
                    message.data.get(GpioUtil.GPIO_CONST_MODE, ''), message.data.get(GpioUtil.GPIO_CONST_NUM, -1),
                    message.data.get(GpioUtil.GPIO_CONST_SIGNAL, ''))
            except Exception, err:
                status_code = Const.SERVER_ERROR_CODE
                status_message = 'failure: %s' % err
            self.reply(status_code, status_message)
        else:
            self.reply(Const.SERVER_ERROR_CODE, 'data is not dict')


class GpioStatusGetHandler(SysHandler):
    """
    GPIO状态获取
    """

    def handle(self, message):
        if isinstance(message.data, dict):
            status_code = Const.SUCCESS_CODE
            status_message = Const.SUCCESS_MESSAGE
            data = GpioUtil.GetGPIOStatus(
                message.data.get(GpioUtil.GPIO_CONST_MODE, ''), message.data.get(GpioUtil.GPIO_CONST_NUM, -1))
            self.reply(status_code, status_message, data)
        else:
            self.reply(Const.SERVER_ERROR_CODE, 'data is not dict')



#!/usr/bin/env python
# -*- coding:utf-8 -*-

import json
import time

import paho.mqtt.client as mqtt

from juice.handler.extend.device.SysGpioHandler import *
from juice.handler.extend.device.SysPropHandler import *
from juice.model.JuiceMessage import JuiceMessage
from juice.util import Util

EVENT_CONNECT_KEY = "sys_connect"
EVENT_DISCONNECT_KEY = "sys_disconnect"
ID = "id"
TYPE = "type"
DATA = "data"
TIME = "time"

sys_handler_dict = {'sys_temp': SysTempGetHandler(), 'sys_volts': SysVoltsGetHandler(),
                    'sys_arm': SysArmGetHandler(), 'sys_gpu': SysGpuGetHandler()
    , 'gpio_get': GpioStatusGetHandler(), 'gpio_init': GpioInitHandler(), 'gpio_set': GpioSetHandler()
                    }


class MqttClient(object):

    def __init__(self, product_code, device_code, secret):
        self.product_code = product_code
        self.device_code = device_code
        self.secret = secret
        self.host = 'mqtt.juicecloud.cn'
        self.port = 2883
        self.prop_report_topic = "/{product_code}/{device_code}/prop/report".format(product_code=self.product_code,
                                                                                    device_code=self.device_code)
        self.prop_up_topic = "/{product_code}/{device_code}/prop/up".format(product_code=self.product_code,
                                                                            device_code=self.device_code)
        self.prop_down_topic = "/{product_code}/{device_code}/prop/down".format(product_code=self.product_code,
                                                                                device_code=self.device_code)
        self.task_up_topic = "/{product_code}/{device_code}/task/up".format(product_code=self.product_code,
                                                                            device_code=self.device_code)
        self.task_down_topic = "/{product_code}/{device_code}/task/down".format(product_code=self.product_code,
                                                                                device_code=self.device_code)
        self.sys_up_topic = "/{product_code}/{device_code}/sys/up".format(product_code=self.product_code,
                                                                          device_code=self.device_code)
        self.sys_down_topic = "/{product_code}/{device_code}/sys/down".format(product_code=self.product_code,
                                                                              device_code=self.device_code)
        self.prop_down_handler = None
        self.task_down_handler = None
        self.sys_down_handler = sys_handler_dict
        self.client = None

    def __str__(self):
        return "%s,%s,%s,%s,%s" % (self.product_code, self.device_code, self.secret,
                                   self.host, self.port)

    def __on_connect(self, client, userdata, flags, rc):
        message = {
            0: 'Connection successful',
            1: 'Connection refused - incorrect protocol',
            2: 'Connection refused - invalid client',
            3: 'Connection refused - server unavailable',
            4: 'Connection refused - bad username or password',
            5: 'Connection refused - not authorised'
        }
        print(message.get(rc, 'Currently unused'))
        self.client.publish(self.sys_up_topic, json.dumps(Util.get_raspberry_sys_info()), qos=0, retain=False)
        self.client.subscribe(self.sys_down_topic)
        print("Subscribe topic: " + self.sys_down_topic)
        self.client.subscribe(self.prop_down_topic)
        print("Subscribe topic: " + self.prop_down_topic)
        self.client.subscribe(self.task_down_topic)
        print("Subscribe topic: " + self.task_down_topic)

    def __on_message(self, client, userdata, msg):
        json_obj = json.loads(msg.payload.decode("utf-8"))
        print "%s: [topic]:[%s] [Received Message]:[%s]" % (
            time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time())), msg.topic, json_obj)
        if ID not in json_obj:
            print "message prop id not exist"
            return
        message_id = json_obj[ID]

        if TYPE in json_obj:
            type_key = json_obj[TYPE]
            message = JuiceMessage(type_key, json_obj.get(DATA, ''), json_obj.get(TIME, int(round(time.time() * 1000))))
            message.id = message_id
            if msg.topic == self.prop_down_topic and type_key in self.prop_down_handler.keys():
                self.prop_down_handler[type_key].execute(self, str(msg.topic), message)
            elif msg.topic == self.task_down_topic and type_key in self.task_down_handler.keys():
                self.task_down_handler[type_key].execute(self, str(msg.topic), message)
            elif msg.topic == self.sys_down_topic and type_key in self.sys_down_handler.keys():
                self.sys_down_handler[type_key].execute(self, str(msg.topic), message)
            else:
                print "%s handler not exist" % type_key
                return

    def __on_disconnect(self, client, userdata, rc):
        pass

    def __on_publish(self, client, userdata, mid):
        pass

    def __on_subscribe(self, client, userdata, mid, granted_qos):
        pass

    def __on_unsubscribe(self, client, userdata, mid):
        pass

    def __on_log(self, client, userdata, level, buf):
        pass

    def __init(self):
        client_id = self.device_code
        client = mqtt.Client('master-%s' % client_id)
        client.username_pw_set(self.device_code, self.secret)
        client.connect(self.host, self.port, 60)
        client.reconnect_delay_set(min_delay=1, max_delay=120)
        # client.on_connect = self.on_connect
        # client.on_message = self.on_message
        # client.on_disconnect = self.on_disconnect
        # client.on_publish = self.on_publish
        # client.on_subscribe = self.on_subscribe
        # client.on_unsubscribe = self.on_unsubscribe
        # client.on_log = self.on_log
        client.on_message = self.__on_message
        client.on_connect = self.__on_connect
        self.client = client

    def __subscriber(self):
        self.client.loop_forever()

    def prop_reply(self, message):
        print "%s: [topic]:[%s] [Send Message]:[%s]" % (
            time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time())), self.prop_up_topic, message)
        self.client.publish(self.prop_up_topic, str(message), qos=0, retain=False)

    def prop_reporter(self, message):
        print "%s: [topic]:[%s] [Send Message]:[%s]" % (
            time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time())), self.prop_report_topic, message)
        self.client.publish(self.prop_report_topic, str(message), qos=0, retain=False)

    def task_reply(self, message):
        print "%s: [topic]:[%s] [Send Message]:[%s]" % (
            time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time())), self.task_up_topic, message)
        self.client.publish(self.task_up_topic, str(message), qos=0, retain=False)

    def sys_reply(self, message):
        print "%s: [topic]:[%s] [Send Message]:[%s]" % (
            time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time())), self.sys_up_topic, message)
        self.client.publish(self.sys_up_topic, str(message), qos=0, retain=False)

    def init(self):
        self.__init()
        self.__subscriber()

    @property
    def prop_message(self):
        return self.prop_down_handler

    @prop_message.setter
    def prop_message(self, handler_dict):
        self.prop_down_handler = handler_dict

    @property
    def task_message(self):
        return self.task_down_handler

    @task_message.setter
    def task_message(self, handler_dict):
        self.task_down_handler = handler_dict



#!/usr/bin/env python
# -*- coding:utf-8 -*-
# @Time   : 2019/5/23 14:31
# @Author : yebing
# @Desc : ==============================================
# Life is Short I Use Python!!!                      ===
# If this runs wrong,don't ask me,I don't know why;  ===
# If this runs right,thank god,and I don't know why. ===
# Maybe the answer,my friend,is blowing in the wind. ===
# ======================================================
# @Project : guozhi
# @FileName: Const.py

SUCCESS_CODE = 200
SUCCESS_MESSAGE = 'success'
SERVER_ERROR_CODE = 500

class Const(object):
    class ConstError(TypeError):
        pass

    class ConstCaseError(ConstError):
        pass

    def __setter__(self, name, value):
        if self.__dict__.has_key(name):
            raise self.ConstError, 'Not allowed change const.{value}'.format(value=name)
        if not name.isupper():
            raise self.ConstCaseError, 'Consts name is not all uppercase'
        self.__dict__[name] = value




